<?php
//  BARMAZ erp system
//  Copyright (c) BARMAZ Group
//  Web: https://BARMAZ.ru/
//  Commercial license https://BARMAZ.ru/article/litsenzionnoe-soglashenie.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 203 (2024-01-07 23:05:31)
// 

//BARMAZ_CORE_INFO

defined('_BARMAZ_VALID') or die("Access denied");

class Measure extends BaseObject{
	private $_loadedMeasures = array();

	//---------- Singleton implementation ------------
	private static $_instance = null;

	public static function createInstance() {
		if (self::$_instance == null) {
			self::$_instance = new self();
		}
	}

	public static function getInstance() {
		self::createInstance();
		return self::$_instance;
	}
	//------------------------------------------------
	public function getMeasure($id){
		if (!array_key_exists($id,$this->_loadedMeasures)) $this->loadMeasure($id);
		return $this->_loadedMeasures[$id];
	}
	private function loadMeasure($id){
		$res=false;
		$db=Database::getInstance();
		$db->setQuery("SELECT * FROM #__measure WHERE meas_id='".$id."'");
		$db->loadObject($res);
		$this->_loadedMeasures[$id]=$res;
	}
	public function getCode($id){
		$meas = $this->getMeasure($id);
		if ($meas) return $meas->meas_code; else return "";
	}
	public function getTitle($id){
		$meas = $this->getMeasure($id);
		if ($meas) return $meas->meas_full_name; else return "";
	}
	public function getShortName($id){
		$meas = $this->getMeasure($id);
		if ($meas) return $meas->meas_short_name; else return "";
	}
	public function getType($id){
		$meas = $this->getMeasure($id);
		if ($meas) return $meas->meas_type; else return 0;
	}
	/**
	 * number of decimal places from site catalog setting
	 * @param int id  ID meausure
	 * @return number
	 */
	public function getNumberDigits($id)
	{
		$type=$this->getType($id);
		//array("0"=>Text::_("Quantity"), "1"=>Text::_("Volume"), "2"=>Text::_("Lenght"), "3"=>Text::_("Time"), "4"=>Text::_("Weight"), 6=>Text::_("Square"));
		switch($type){
			case 4:
				return catalogConfig::$weight_digits;
				break;
			case 3:
				return 0;
				break;
			case 2:
			case 6:
				return catalogConfig::$size_digits;
				break;
			case 1:
				return catalogConfig::$volume_digits;
				break;
			case 0:
			default:
				return catalogConfig::$quantity_digits;
				break;
		}
	}	
	public function getKoeff($id){
		$meas = $this->getMeasure($id);
		if ($meas) return $meas->meas_kf; else return 0;
	}
	public function convert($quantity, $in_id, $out_id){
		$meas_in=$this->getMeasure($in_id);
		$meas_out=$this->getMeasure($out_id);
		if ($meas_in && $meas_out && ($meas_in->meas_type == $meas_out->meas_type)){
			if (!$meas_out->meas_kf) return 0;
			return $quantity * $meas_in->meas_kf / $meas_out->meas_kf;
		} else return 0;
	}
	
}
